<?php
/*
 * @package userport
 * @copyright 2008-2011 Parvus
 * @license http://www.gnu.org/licenses/gpl-3.0.html
 * @link http://joomlacode.org/gf/project/userport/
 * @author Parvus
 *
 * userport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * userport is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with userport. If not, see <http://www.gnu.org/licenses/>.
 */
defined( '_JEXEC' ) or die( 'Restricted access' );

$user = & JFactory::getUser();
$user->authorize( 'com_users', 'manage' ) or die( JText::_( 'ALERTNOTAUTH' ) );

require_once( JApplicationHelper::getPath( 'admin_html' ) );
require_once( JApplicationHelper::getPath( 'helper' ) );
jimport( 'joomla.user.helper' );

$lang =& JFactory::getLanguage();
$lang->load( 'com_users', JPATH_ADMINISTRATOR );

$css = JURI::base().'components/com_userport/userport.css';
$document = &JFactory::getDocument();
$document->addStyleSheet( $css, 'text/css', null, array() );

function _UpdateUser_name( & $user, & $userData, $settings, & $log, & $userLog )
{
  if ( ( !/*NOT*/isset( $userData->name ) ) or ( JString::strlen( $userData->name ) == 0 ) )
  {
    $userData->name = $user->name;
  }
  if ( $user->name != $userData->name )
  {
    $userLog[] = JText::sprintf( 'INFO_NAME_UPDATED', $user->name, $userData->name );
    $user->name = $userData->name;
  }
}

function _UpdateUser_email( & $user, & $userData, $settings, & $log, & $userLog )
{
  if ( ( !/*NOT*/isset( $userData->email ) ) or ( JString::strlen( $userData->email ) == 0 ) )
  {
    $userData->email = $user->email;
  }
  if ( $user->email != $userData->email )
  {
    $userLog[] = JText::sprintf( 'INFO_EMAIL_UPDATED', $user->email, $userData->email );
    $user->email = $userData->email;
  }
}

function _UpdateUser_password( & $user, & $userData, $settings, & $log, & $userLog )
{
  $unencryptedPassword = '********';
  if ( isset( $userData->password ) and ( JString::strlen( $userData->password ) > 0 ) )
  {
    if ( $settings[ 'passwords_are_encoded' ] )
    {
      /* $userData->password is already encrypted. */
    }
    else
    {
      $unencryptedPassword = $userData->password;
      /* Change the password, but try to retain the salt value of the existing
       * password. It might be that the same password is given as new value.
       * When the same salt is used, the resulting hashed password string will
       * be exactly the same.
       */
      $existingPasswordParts = explode(':', $user->password);
      if ( count( $existingPasswordParts ) == 2 )
      {
        $salt = $existingPasswordParts[1];
      }
      else
      {
        $salt = JUserHelper::genRandomPassword( 32 );
      }
      $crypt = JUserHelper::getCryptedPassword( $unencryptedPassword, $salt );
      $userData->password = $crypt . ':' . $salt;
    }
    if ( $user->password != $userData->password )
    {
      $userLog[] = JText::sprintf( 'INFO_PASSWORD_UPDATED', $unencryptedPassword );
      $user->password = $userData->password;
    }
  }
  $userData->password = $unencryptedPassword;
}

function _UpdateUser_usertype( & $user, & $userData, $settings, & $log, & $userLog )
{
  if ( in_array( $settings[ 'usertype' ], JUserportHelper::UserTypes() ) )
  {
    $userType = $settings[ 'usertype' ];
  }
  else if ( isset( $userData->usertype ) and ( JString::strlen( $userData->usertype ) > 0 ) )
  {
    $userType = $userData->usertype;
  }
  else
  {
    /* Use the current user's value, so that an update will not be done. */
    $userType = $user->usertype;
  }

  if ( !/*NOT*/in_array( $userType, JUserportHelper::UserTypes() ) )
  {
    /* Start with a valid value: fix the database value. */
    $userType = $settings[ 'default_usertype' ];
  }

  if ( $user->usertype != $userType )
  {
    if ( in_array( $userType, JUserportHelper::UserTypes() ) )
    {
      $userLog[] = JText::sprintf( 'INFO_USERTYPE_UPDATED', $user->usertype, $userType );
      $user->usertype = $userType;
      $user->gid = JUserportHelper::Gid( $user->usertype );
    }
    else
    {
      $log[] = array( 'line_warning',
          JText::sprintf( 'WARNING_INVALID_USERTYPE', $userType, $userData->username ) );
    }
  }
}

function _UpdateUser_block( & $user, & $userData, $settings, & $log, & $userLog )
{
  if ( $settings[ 'block_activate' ] == 'yes_yes' )
  {
    $block = 1;
    $explicitlySetActivationString = false;
    $explicitlyClearActivationString = true;
  }
  else if ( $settings[ 'block_activate' ] == 'yes_no' )
  {
    $block = 1;
    $explicitlySetActivationString = true;
    $explicitlyClearActivationString = false;
  }
  else if ( $settings[ 'block_activate' ] == 'no_yes' )
  {
    $block = 0;
    $explicitlySetActivationString = false;
    $explicitlyClearActivationString = true;
  }
  else
  {
    if ( isset( $userData->block ) and ( JString::strlen( $userData->block ) > 0 ) )
    {
      $block = $userData->block;
    }
    else
    {
      $block = $user->block;
    }
    $explicitlySetActivationString = false;
    $explicitlyClearActivationString = false;
  }

  /*
   * ActivationString:
   *
   * Option:    |  New UserData:                |   Existing Data:    |
   * Clear Set  |  Not-Given  Non-Empty  Empty  |   Non-Empty  Empty  |  What to do?
   * -----------+-------------------------------+---------------------+-------------------------
   *   Y    N   |   -          -          -     |    -          -     |   Clear (A)
   * -----------+-------------------------------+---------------------+-------------------------
   *   N    Y   |   N          Y          N     |    -          -     |   Set, use UserData (B)
   *   N    Y   |   -          N          -     |    Y          N     |   Retain current status (C)
   *   N    Y   |   -          N          -     |    N          Y     |   Set, generate (D)
   * -----------+-------------------------------+---------------------+-------------------------
   *   N    N   |   Y          N          N     |    -          -     |   Retain current status (E)
   *   N    N   |   N          Y          N     |    -          -     |   Set, use UserData (F)
   *   N    N   |   N          N          Y     |    -          -     |   Clear (G)
   */

  if ( $explicitlyClearActivationString )
  {
    /* Clear: Y - (A) */
    $activation = '';
  }
  else if ( $explicitlySetActivationString )
  {
    if ( isset( $userData->activation ) and ( JString::strlen( $userData->activation ) > 0 ) )
    {
      /* Set: Y, UserData Non-Empty: Y - (B) */
      $activation = $userData->activation;
    }
    else if ( JString::strlen( $user->activation ) > 0 )
    {
      /* Set: Y, UserData Empty: Y, Existing Data: Y - (C) */
      $activation = $user->activation;
    }
    else
    {
      /* Set: Y, Non-Empty: N - (D) */
      $activation = JUtility::getHash( JUserHelper::genRandomPassword() );
    }
  }
  else if ( isset( $userData->activation ) )
  {
    if ( JString::strlen( $userData->activation ) > 0 )
    {
    	/* Clear: N, Set: N, Non-Empty: Y - (F) */
      $activation = $userData->activation;
    }
    else
    {
      /* Clear: N, Set: N, Empty: Y - (G) */
      $activation = '';
    }
  }
  else
  {
    /* Clear: N, Set: N, Not-Given: Y - (E) */
    $activation = $user->activation;
  }
  if ( ( !/*NOT*/$block ) and ( JString::strlen( $activation ) > 0 ) )
  {
    $block = 1;
    $userLog[] = JText::sprintf( 'WARNING_FIELD_CHANGED_TO_MATCH_ACTIVATION', 'block' );
  }

  if ( JString::strlen( $activation ) > 0 )
  {
    /* An activation string can only be used if the lastvisitDate is not yet set.
     * Under the assumption that the dates are checked and updated @b after this function,
     * we can set or override the lastvisitDate here, ensuring the activation url will work.
     */
    $userData->lastvisitDate = "0000-00-00 00:00:00";
  }

  $explicitlyLogBlockedStatus = false;
  if ( $user->activation != $activation )
  {
    if ( $activation )
    {
      if ( $user->activation )
      {
        $userLog[] = JText::sprintf( 'INFO_ACTIVATION_UPDATED', $activation );
      }
      else
      {
        $userLog[] = JText::sprintf( 'INFO_ACTIVATION_SET', $activation );
      }
    }
    else
    {
      $userLog[] = JText::_( 'INFO_ACTIVATED' );
      $explicitlyLogBlockedStatus = true;
    }
    $user->activation = $activation;
  }

  $user->block = (int)$user->block;
  $block = (int)$block;
  if ( ( $user->block != $block ) or $explicitlyLogBlockedStatus )
  {
    if ( JString::strlen( $user->activation ) == 0 )
    {
      $userLog[] = $block ? JText::_( 'INFO_BLOCKED' ) : JText::_( 'INFO_UNBLOCKED' );
    }
    else
    {
      /* No need to tell the user is blocked. An activation string is set;
       * being blocked now is implied with that.
       */
    }
    $user->block = $block;
  }
}

function _UpdateUser_sendEmail( & $user, & $userData, $settings, & $log, & $userLog )
{
  if ( ( $settings[ 'sendEmail' ] !== 'default' )
    and ( ( $settings[ 'sendEmail' ] == 0 ) or ( $settings[ 'sendEmail' ] == 1 ) ) )
  {
    $sendEmail = $settings[ 'sendEmail' ];
  }
  else if ( isset( $userData->sendEmail ) and ( JString::strlen( $userData->sendEmail ) > 0 ) )
  {
    $sendEmail = $userData->sendEmail;
  }
  else
  {
    $sendEmail = $user->sendEmail;
  }
  $user->sendEmail = (int)$user->sendEmail;
  $sendEmail = (int)$sendEmail;
  if ( $user->sendEmail != $sendEmail )
  {
    $userLog[] = $sendEmail ? JText::_( 'INFO_SENDEMAIL_ON' ) : JText::_( 'INFO_SENDEMAIL_OFF' );
    $user->sendEmail = (int)$sendEmail;
  }
}

function _UpdateUser_registerDate( & $user, & $userData, $settings, & $log, & $userLog )
{
  if ( JString::strlen( $settings[ 'registerDate' ] ) > 0 )
  {
    $registerDate = $settings[ 'registerDate' ];
  }
  else if ( isset( $userData->registerDate ) and ( JString::strlen( $userData->registerDate ) > 0 ) )
  {
    $registerDate = $userData->registerDate;
  }
  else
  {
    $registerDate = $user->registerDate;
  }

  /* Be sure the string is properly formatted. */
  $registerDate =& JFactory::getDate( $registerDate );
  $registerDate = $registerDate->toFormat();

  if ( $user->registerDate != $registerDate )
  {
    $userLog[] = JText::sprintf( 'INFO_REGISTRATION_DATE_UPDATED', $user->registerDate, $registerDate );
    $user->registerDate = $registerDate;
  }
}

function _UpdateUser_lastvisitDate( & $user, & $userData, $settings, & $log, & $userLog )
{
  if ( JString::strlen( $user->activation ) > 0 )
  {
    /* No matter what the value of lastvisitDate is in $user or $userData,
     * it must become the reset value.
     */
    $lastvisitDate = "0000-00-00 00:00:00";
    if ( ( $settings[ 'lastvisitDate' ] != $lastvisitDate )
        or ( isset( $userData->lastvisitDate ) and ( $userData->lastvisitDate != $lastvisitDate ) ) )
    {
      $userLog[] = JText::sprintf( 'WARNING_FIELD_CHANGED_TO_MATCH_ACTIVATION', 'lastvisitDate' );
    }
    $user->lastvisitDate = $lastvisitDate;
  }
  else
  {
    if ( JString::strlen( $settings[ 'lastvisitDate' ] ) > 0 )
    {
      $lastvisitDate = $settings[ 'lastvisitDate' ];
    }
    else if ( isset( $userData->lastvisitDate ) and ( JString::strlen( $userData->lastvisitDate ) > 0 ) )
    {
      $lastvisitDate = $userData->lastvisitDate;
    }
    else
    {
      $lastvisitDate = $user->lastvisitDate;
    }

    /* Be sure the string is properly formatted. */
    if ( $lastvisitDate != "0000-00-00 00:00:00" )
    {
      $lastvisitDate =& JFactory::getDate( $lastvisitDate );
      $lastvisitDate = $lastvisitDate->toFormat();
    }

    if ( $user->lastvisitDate != $lastvisitDate )
    {
      $userLog[] = JText::sprintf( 'INFO_LAST_VISIT_DATE_UPDATED', $user->lastvisitDate, $lastvisitDate );
      $user->lastvisitDate = $lastvisitDate;
    }
  }
}

function _UpdateUser_params( & $user, & $userData, $settings, & $log, & $userLog )
{
  if ( JString::strlen( $settings[ 'params' ] ) > 0 )
  {
    $params = $settings[ 'params' ];
  }
  else if ( isset( $userData->params ) and ( JString::strlen( $userData->params ) > 0 ) )
  {
    $params = $userData->params;
  }
  else
  {
    $params = $user->params;
  }
  if ( JString::strpos( $params, '|' ) !== FALSE )
  {
    $params = JString::str_ireplace( '|', "\n", $params );
  }
  if ( $user->params != $params )
  {
    foreach ( explode( "\n", $params ) as $param )
    {
      if ( JString::strpos( $param, '=' ) === FALSE )
      {
        $param = $param . '=';
      }
      $param = explode( '=', $param, 2 );
      if ( JString::strlen( $param[ 0 ] ) > 0 )
      {
        $user->setParam( $param[ 0 ], $param[ 1 ] );
      }
    }
    $userLog[] = JText::sprintf( 'INFO_PARAMS_UPDATED', $user->params, $params );
  }
}

function _UpdateUser( $task, $userId, $userData, $settings, & $log )
{
  $user = JUser::getInstance( $userId );
  if ( in_array( $user->gid, JUserportHelper::Gids() ) )
  {
    /* Update the existing information where possible and allowed. */
    $userLog = array();

    foreach ( JUserportHelper::Fields( 'import' ) as $field )
    {
      $function = '_UpdateUser_' . $field;
      if ( function_exists( $function ) )
      {
        call_user_func_array( $function, array( & $user, & $userData, $settings, & $log, & $userLog ) );
      }
    }

    /* Store the updated information in the database. */
    if ( count( $userLog ) == 0 )
    {
      $log[] = array( 'line_ignored',
        JText::sprintf( 'INFO_EXISTING_USER_WAS_ALREADY_UP_TO_DATE', '['.$userData->username.'] '.$userData->name ) );
    }
    else
    {
      if ( $settings[ 'dry_run' ] )
      {
        /* Do not make any modification to the database. */
        $success = true;
      }
      else
      {
        $success = $user->save( false );
      }

      if ( $success )
      {
        $user->password = $userData->password; /* Set the unencrypted password. */
        $notified = false;
        if ( $settings[ 'notify_user' ] )
        {
          $notified = JUserportHelper::NotifyUser( $task, $user, $settings );
        }
        $string = JText::sprintf( 'INFO_EXISTING_USER_UPDATED', '['.$userData->username.'] '.$userData->name );
        if ( $notified )
        {
          $string .= ' ' . JText::_( 'INFO_USER_NOTIFIED' );
        }
        if ( isset( $userData->superfluous ) and ( JString::strlen( $userData->superfluous ) > 0 ) )
        {
          $userLog[] = JText::sprintf( 'WARNING_SUPERFLUOUS_FIELDS', $userData->superfluous );
        }
        $string .= '<br/>' . implode( '<br/>', $userLog );
        $log[] = array( 'line_success', $string );
      }
      else
      {
        $string = JText::sprintf( 'ERROR_COULD_NOT_UPDATE_EXISTING_USER', '['.$userData->username.'] '.$userData->name ) . ' ' . JText::_( $user->getError() );
        $log[] = array( 'line_error', $string );
      }
    }
  }
  else
  {
    /* Only allow changes to users who have front-end-only rights. */
    $log[] = array( 'line_ignored',
      JText::sprintf( 'WARNING_USER_RIGHTS_PROHIBIT_CHANGE', '['.$userData->username.'] '.$userData->name  ) );
  }
}

/**
 * @param $task
 * @param $userData $userData->username must exist, this is not checked for.
 * @param $settings
 * @param $log
 */
function _AddUser($task, $userData, $settings, & $log)
{
  $userLog = array();
  $success = true;
  $passwordIsEncoded = $settings[ 'passwords_are_encoded' ];

  if ( !/*NOT*/isset( $userData->name ) or ( JString::strlen( $userData->name ) == 0 ) )
  {
    $userData->name = $userData->username;
  }

  if ( !/*NOT*/isset( $userData->email ) or ( JString::strlen( $userData->email ) == 0 ) )
  {
    $log[] = array( 'line_error', JText::sprintf( 'ERROR_NO_EMAIL_FOUND_FOR_NEW_USER', '['.$userData->username.'] '.$userData->name ) );
    $success = false;
  }

  if ( !/*NOT*/isset( $userData->password ) or JString::strlen( $userData->password ) == 0 )
  {
    if ( $settings[ 'auto_generate_passwords' ] )
    {
      $userData->password = JUserHelper::genRandomPassword();
      $passwordIsEncoded = false;
      $userLog[] = JText::sprintf( 'INFO_PASSWORD_GENERATED_FOR_NEW_USER', $userData->password, '['.$userData->username.'] '.$userData->name );
    }
    else
    {
      $log[] = array( 'line_error', JText::sprintf( 'ERROR_NO_PASSWORD_FOUND_FOR_NEW_USER', '['.$userData->username.'] '.$userData->name ) );
      $success = false;
    }
  }

  if ( $success )
  {
    $newUser = new JUser();

    if ( isset( $userData->superfluous ) and ( JString::strlen( $userData->superfluous ) > 0 ) )
    {
      $userLog[] = JText::sprintf( 'WARNING_SUPERFLUOUS_FIELDS', $userData->superfluous );
    }

    /* First fill in some default fallback values. */
    $newUser->usertype = $settings[ 'default_usertype' ];
    $newUser->block = 0;
    $newUser->activation = '';
    $newUser->lastvisitDate = "0000-00-00 00:00:00";

    /* Next fill in the given values. */
    foreach( get_class_vars( 'JUser' ) as $var => $value )
    {
      if ( isset( $userData->$var ) and ( JString::strlen( $userData->$var ) > 0 ) )
      {
        $newUser->$var = $userData->$var;
      }
    }
    if ( !/*not*/in_array( $newUser->usertype, JUserportHelper::UserTypes() ) )
    {
      $userLog[] = JText::sprintf( 'WARNING_INVALID_USERTYPE', $newUser->usertype, $userData->username );
      $newUser->usertype = $settings[ 'default_usertype' ];
    }

    /* Next possibly override with some administrator-configured values. */
    if ( $settings[ 'usertype' ] != 'default' )
    {
      $newUser->usertype = $settings[ 'default_usertype' ];
    }
    if ( $settings[ 'block_activate' ] == 'yes_yes' )
    {
      $newUser->block = 1;
      $explicitlySetActivationString = false;
    }
    else if ( $settings[ 'block_activate' ] == 'yes_no' )
    {
      $newUser->block = 1;
      $newUser->lastvisitDate = "0000-00-00 00:00:00";
      $explicitlySetActivationString = true;
    }
    else if ( $settings[ 'block_activate' ] == 'no_yes' )
    {
      $newUser->block = 0;
      $explicitlySetActivationString = false;
    }
    else
    {
      /* Retain value of $newUser->block */
      $explicitlySetActivationString = false;
    }
    if ( ( $explicitlySetActivationString ) && ( JString::strlen( $newUser->activation ) == 0 ) )
    {
      $newUser->activation = JUtility::getHash( JUserHelper::genRandomPassword() );
    }
    if ( JString::strlen( $newUser->activation ) > 0 )
    {
      $userLog[] = JText::sprintf( 'INFO_ACTIVATION_SET', $newUser->activation );
      if ( !/*NOT*/$newUser->block )
      {
        $newUser->block = 1;
        $userLog[] = JText::sprintf( 'WARNING_FIELD_CHANGED_TO_MATCH_ACTIVATION', 'block' );
      }
      if ( $newUser->lastvisitDate != "0000-00-00 00:00:00" )
      {
        $newUser->lastvisitDate = "0000-00-00 00:00:00";
        $userLog[] = JText::sprintf( 'WARNING_FIELD_CHANGED_TO_MATCH_ACTIVATION', 'lastvisitDate' );
      }
    }
    $newUser->gid = JUserportHelper::Gid( $newUser->usertype );

    if ( !/*NOT*/isset( $userData->registerDate ) or !/*NOT*/$userData->registerDate )
    {
      $date =& JFactory::getDate();
      $newUser->registerDate = $date->toMySQL();
    }

    if ( $passwordIsEncoded )
    {
      $unencryptedPassword = '********';
      /* $user->password is already encrypted. */
    }
    else
    {
      $unencryptedPassword = $userData->password;
      $salt = JUserHelper::genRandomPassword( 32 );
      $crypt = JUserHelper::getCryptedPassword( $unencryptedPassword, $salt );
      $newUser->password = $crypt . ':' . $salt;
    }
    $userData->password = $unencryptedPassword;

    if ( $settings[ 'dry_run' ] )
    {
      /* Do not make any modification to the database. */
      $success = true;
    }
    else
    {
      $success = $newUser->save( false );
    }

    if ( $success )
    {
      $newUser->password = $userData->password; /* Set the unencrypted password. */
      $notified = false;
      if ( $settings[ 'notify_user' ] )
      {
        $notified = JUserportHelper::NotifyUser( $task, $newUser, $settings );
      }
      $string = JText::sprintf( 'INFO_NEW_USER_ADDED', '['.$userData->username.'] '.$userData->name );
      if ( $notified )
      {
        $string .= ' ' . JText::_( 'INFO_USER_NOTIFIED' );
      }
      if ( count( $userLog ) > 0 )
      {
        $string .= '<br/>' . implode( '<br/>', $userLog );
      }
      $log[] = array( 'line_success', $string );
    }
    else
    {
      $string = JText::sprintf( 'ERROR_COULD_NOT_ADD_NEW_USER', '[' . $userData->username . '] ' . $userData->name ) . ' ' . JText::_( $newUser->getError() );
      $log[] = array( 'line_error', $string );
    }
  }
}

function _DeleteUser( $task, $userId, $userData, $settings, & $log )
{
  $delete = false;
  $success = true;
  $user = JUser::getInstance( $userId );
  $matched = 0;
  if ( in_array( $user->gid, JUserportHelper::Gids() ) )
  {
    /* Check the name and email. */
    $userLog = array();
    if ( isset( $userData->name ) and JString::strlen( $userData->name ) > 0 )
    {
      $name = $user->name;
      if ( $name != $userData->name)
      {
        $userLog[] = JText::sprintf( 'WARNING_NAME_MISMATCH', $userData->name, $name );
      }
    }
    if ( isset( $userData->email ) and JString::strlen( $userData->email ) > 0 )
    {
      $email = $user->email;
      if ( $email != $userData->email )
      {
        $userLog[] = JText::sprintf( 'WARNING_EMAIL_MISMATCH', (string)$userData->email, $email);
      }
    }

    /* If userLog is still empty, no mismatches have been discovered.
     * Only in that case we proceed and delete.
     */
    if ( count( $userLog ) == 0)
    {
      if ( $settings[ 'dry_run' ] )
      {
        /* Do not make any modification to the database. */
        $success = true;
      }
      else
      {
        $success = $user->delete();
      }

      if ( $success )
      {
        $user->password = '********'; /* Set the unencrypted password. */
        $notified = false;
        if ( $settings[ 'notify_user' ] )
        {
          $notified = JUserportHelper::NotifyUser( $task, $user, $settings );
        }
        $string = JText::sprintf( 'INFO_EXISTING_USER_DELETED', '['.$userData->username.'] ' . $userData->name );
        if ( $notified )
        {
          $string .= ' ' . JText::_( 'INFO_USER_NOTIFIED' );
        }
        $log[] = array( 'line_success', $string );
      }
      else
      {
        $string = JText::sprintf( 'ERROR_COULD_NOT_DELETE_EXISTING_USER', '['.$userData->username.'] '.$userData->name ) . ' ' . JText::_( $user->getError() );
        $log[] = array( 'line_error', $string );
      }
    }
    else
    {
      $log[] = array( 'line_error',
        JText::sprintf( 'WARNING_USER_NOT_DELETED', '['.$user->username.'] ' . $user->name)
        . '<br/>' . implode( '<br/>', $userLog ) );
    }
  }
  else
  {
    /* Only allow changes to users who have front-end-only rights. */
    $log[] = array( 'line_ignored',
        JText::sprintf( 'WARNING_USER_RIGHTS_PROHIBIT_CHANGE', '['.$userData->username.'] '.$userData->name  ) );
  }
}

/******************************************************************************/

function _Userport_Import( $option, $task, $settings, & $log )
{
  if ( JString::strlen( JString::trim( $settings[ 'user_list' ] ) ) == 0 )
  {
    $log[] = array( 'warning', JText::_( 'WARNING_NO_IMPORT_TEXT' ) );
  }
  else
  {
    $headerString = '';
    $headerData = null;

    if ( $settings[ 'dry_run' ] )
    {
      $log[] = array( 'warning', JText::_( 'INFO_DRY_RUN_MODE' ) );
    }

    $startTime = microtime( true );
    $maxExecutionTime = ini_get('max_execution_time');
    if ( ( $maxExecutionTime <= 0 ) or ( $maxExecutionTime > 30 ) )
    {
      /* Even when execution time is unrestricted or very large, userport
       * restricts itself. This way, massive imports don't take minutes
       * during which no feedback is given to the user.
       */
      $maxExecutionTime = 30;
    }
    $latestEndTime = max( $startTime + 5/* seconds */, $startTime + $maxExecutionTime - 5/* seconds */ );

    if ( intval( $settings[ 'parsing_index' ] ) > 0 )
    {
      $log[] = array( 'info', JText::sprintf( 'INFO_CONTINUED_PARSING',  $settings[ 'parsing_index' ] ) );
    }

    $userStrings = explode( "\n", $settings[ 'user_list' ] );

    /* Reset some variables. They will be set again below if further batch processing is needed. */
    $settings[ 'user_list' ] = '';
    unset( $GLOBALS[ 'userport_next_batch_needs_processing' ] );
    $fieldSeparator = $settings[ 'field_separator' ]; /* Will be overwritten when parsing the header. Just bring it to existence here. */
    $noImportText = true;
    for ( $i = 0; $i < count( $userStrings ); $i++ )
    {
      $userString = $userStrings[ $i ];
      $userString = JString::trim( $userString );
      if ( JString::strlen( $userString ) == 0 )
      {
        /* Ignore without a warning. */
      }
      else if ( $headerData )
      {
        /* At least one non-empty non-header line is present. */
        $noImportText = false;
        $userData = JUserportHelper::CsvStringToObject( $userString, $headerData, $fieldSeparator, $settings, $log );
        /* Camel-case the usertype. */
        if ( isset( $userData->usertype ) and ( JString::strlen( $userData->usertype ) > 0 ) )
        {
          $userData->usertype = JString::ucfirst( JString::strtolower( $userData->usertype ) );
        }
        if ( isset( $userData->username ) and ( JString::strlen( $userData->username ) > 0 ) )
        {
          $userId = JUserHelper::getUserId( $userData->username );
          switch( $task )
          {
            case 'add':
              if ( $userId > 0 )
              {
                /* Updating or deleting existing users is not requested. */
                $log[] = array( 'line_ignored', JText::sprintf( 'WARNING_USER_ALREADY_EXISTS', $userData->username ) );
              }
              else
              {
                _AddUser( 'add', $userData, $settings, $log );
              }
              break;

            case 'update':
              if ( $userId > 0 )
              {
                _UpdateUser( 'update', $userId, $userData, $settings, $log );
              }
              else
              {
                /* Adding new users is not requested. */
                $log[] = array( 'line_ignored', JText::sprintf( 'WARNING_USER_DOES_NOT_EXIST', $userData->username ) );
              }
              break;

            case 'delete':
              if ( $userId > 0 )
              {
                _DeleteUser( 'delete', $userId, $userData, $settings, $log );
              }
              else
              {
                /* Adding new users is not requested. */
                $log[] = array( 'line_ignored', JText::sprintf( 'WARNING_USER_DOES_NOT_EXIST', $userData->username ) );
              }
              break;

            default:
              $log[] = array( 'line_error', JText::sprintf( 'ERROR_INVALID_TASK', $task) );
              break;
          }
        }
        else
        {
          $log[] = array( 'line_error', JText::sprintf( 'ERROR_USERNAME_NOT_SET', $userString) );
        }

        $currentTime = microtime( true );
        if ( $currentTime >= $latestEndTime )
        {
          /* Further batch processing is needed. Set the needed variables correctly. */
          $log[] = array( 'warning', JText::_( 'WARNING_MAXIMUM_EXECUTION_TIME_REACHED' ) );
          $userStrings = array_slice( $userStrings, $i + 1 );
          $settings[ 'parsing_index' ] += $i;
          $settings[ 'user_list' ] = $headerString . "\n" . implode( "\n", $userStrings );
          $GLOBALS[ 'userport_next_batch_needs_processing' ] = true;
          break; /* Exit the for loop. */
        }
      }
      else
      {
        $headerString = $userString;
        $fieldSeparator = JUserportHelper::DetermineFieldSeparator( $headerString );
        $log[] = array( 'info', JText::sprintf( 'INFO_DETERMINED_FIELD_SEPARATOR', $fieldSeparator ) );
        $headerData = JUserportHelper::CsvStringToObject( $headerString, null, $fieldSeparator, $settings, $log );
        $headerData = array_values( (array) $headerData );
        foreach ( $headerData as $field )
        {
          if ( ( JString::strlen( $field ) > 0 ) and ( !/*NOT*/in_array( $field, JUserportHelper::Fields( 'import' ) ) ) )
          {
            $log[] = array( 'warning', JText::sprintf( 'WARNING_UNKNOWN_HEADER_FIELD', $field ) );
          }
        }
        $headerIsRead = true;
      }
    }

    if ( $noImportText )
    {
      $log[] = array( 'warning', JText::_( 'WARNING_NO_IMPORT_TEXT' ) );
    }
  }

  $title = array
  ( 'add' => JText::_( 'FORM_USERS_ADDED' ).' ',
    'update' => JText::_( 'FORM_USERS_UPDATED' ).' ',
    'delete' => JText::_( 'FORM_USERS_DELETED' ).' ' );
  JUserportHtml::PrintLog( $title[ $task ] . JText::_( 'FORM_LOG' ), $log );
  JUserportHtml::ShowEndResult( $option, $task, $settings );
}

function _Userport_Delete( $option, $task, $settings, & $log )
{
  if ( in_array( '0', array( $settings[ 'confirmation1' ], $settings[ 'confirmation2' ], $settings[ 'confirmation3' ]) ) )
  {
    $log[] = array( 'error', JText::_( 'ERROR_ABORTED_UNCONFIRMED_DELETE' ) );
    _Userport_ShowAbortedWindow( $option, $task, $settings, $log );
  }
  else
  {
    _Userport_Import( $option, $task, $settings, $log );
  }
}

function _Userport_ShowEditWindow( $option, $task, $settings, & $log )
{
  $initialText = '';
  switch ( $settings[ 'initial_text' ] )
  {
    case 'empty':
      break;

    case 'file':
      $initialText = JUserportHelper::GetUploadedFile( $log, $settings[ 'remove_quotes' ] );
      break;

    case 'current_users':
    default:
      $initialText = JUserportHelper::ObjectToCsvString( null, $settings[ 'field_separator' ], $settings, $log );
      $userList = JUserportHelper::GetUserList( $task, $settings, $log );
      foreach ( $userList as $userData )
      {
        $initialText .= JUserportHelper::ObjectToCsvString( $userData, $settings[ 'field_separator' ], $settings, $log );
      }
      break;
  }

  /* @c $initialText may still be empty, both when 'empty' (empty for sure) and
   * 'file' (if the file was empty) is chosen.
   * In those cases, ensure a full header is present.
   */
  if ( JString::strlen( $initialText ) == 0 )
  {
    $whichFields = $settings[ 'which_fields' ];
    $settings[ 'which_fields' ] = 'import';
    $initialText = JUserportHelper::ObjectToCsvString( null, $settings[ 'field_separator' ], $settings, $log );
    $settings[ 'which_fields' ] = $whichFields;
  }
  JUserportHtml::PrintLog( JText::_( 'FORM_LOG' ), $log );
  JUserportHtml::ShowTextarea( $option, $task, JText::_( 'LABEL_USERS_TO_IMPORT' ), $initialText );
}

function _Userport_ShowAbortedWindow( $option, $task, $settings, & $log )
{
  $logTitle = JText::_( 'FORM_ABORTED' ).' '.JText::_( 'FORM_LOG' );
  JUserportHtml::PrintLog( $logTitle, $log );
  JUserportHtml::DummyForm( $option, $task );
}

function _Userport_ShowOptions( $option, $task, $settings, & $log )
{
  $headerString = $settings[ 'user_list' ];
  $length = JString::strpos( $headerString, "\n" );
  if ( $length )
  {
    $headerString = JString::substr( $headerString, 0, $length );
  }
  $fieldSeparator = JUserportHelper::DetermineFieldSeparator( $headerString );
  $log[] = array( 'info', JText::sprintf( 'INFO_DETERMINED_FIELD_SEPARATOR', $fieldSeparator ) );
  $headerData = JUserportHelper::CsvStringToObject( $headerString, null, $fieldSeparator, $settings, $log );
  $headerData = array_values( (array) $headerData );
  foreach ( $headerData as $field )
  {
    if ( ( JString::strlen( $field ) > 0 ) and ( !/*NOT*/in_array( $field, JUserportHelper::Fields( 'import' ) ) ) )
    {
      $log[] = array( 'warning', JText::sprintf( 'WARNING_UNKNOWN_HEADER_FIELD', $field ) );
    }
  }

  $title = array
  ( 'show-options-for-add' => JTEXT::_( 'LABEL_ADD_OPTIONS' ),
    'show-options-for-update' => JTEXT::_( 'LABEL_UPDATE_OPTIONS' ),
    'show-options-for-delete' => JTEXT::_( 'LABEL_DELETE_OPTIONS' ) );
  JUserportHtml::ShowOptions( $option, $task, $title[ $task ], $settings );
  JUserportHtml::PrintLog( JText::_( 'FORM_LOG' ), $log );
}

function _Userport_ChooseInitialText( $option, $task, $settings, & $log )
{
  $showRemoveQuotesOption = false;
  if ( !/*NOT*/ function_exists( 'str_getcsv' ) )
  {
    $log[] = array( 'warning', JText::sprintf( 'WARNING_LIMITED_CSV_IMPORT_FUNCTIONALITY', 'str_getcsv' ) );
    $showRemoveQuotesOption = true;
  }
  JUserportHtml::ShowChooseInitialTextForm( $option, $task, JTEXT::_( 'LABEL_CHOOSE_INITIAL_TEXT' ), $settings, $showRemoveQuotesOption );
}

function _Userport_Export( $option, $task, $settings, & $log )
{
  $userList = JUserportHelper::GetUserList( $task, $settings, $log );
  $csvExportString = JUserportHelper::ObjectToCsvString( null, $settings[ 'field_separator' ], $settings, $log );
  if ( count( $userList ) > 0 )
  {
    foreach ( $userList as $userData )
    {
      $csvExportString .= JUserportHelper::ObjectToCsvString( $userData, $settings[ 'field_separator' ], $settings, $log );
    }
  }
  $logTitle = JText::_( 'FORM_EXPORTED_USERS' ).' '.JText::_( 'FORM_LOG' );

  JUserportHtml::PrintLog( $logTitle, $log );
  JUserportHtml::ShowExportedUsers( $option, $task, $csvExportString );
}

function _Userport_Download( $option, $task, $settings, & $log )
{
  $filename = 'users-' . date( 'Ymd-Hi' ) . '.csv';
  JUserportHelper::CreateDownloadFile( $settings[ 'user_list' ], $filename );
}

function _Userport_Start( $option, $task, $settings, & $log )
{
  JUserportHtml::PrintHelp();
  JUserportHtml::DummyForm( $option, $task );
}

/* ********************************************************************** */

/**
 * userport's 'main function'. Each time this component is loaded, this
 * function is the first function called. It transfers control to a specific
 * task handling function
 * @param $option A string. Joomla terminology for the component that is
 * currently run.
 * @param $task A string. The task to execute.
 * @param $settings A named array of settings to be used.
 * @param $log An array of arrays. Each unnamed index is an array with two
 * enforced unnamed indices: the first (index 0) leads to a string indicating
 * the type, the second (index 1) leads to a string with the actual message.
 * @return void
 */
function _Userport_HandleTask( $option, $task, $settings, & $log )
{
  $tasks = array(
    '' => 'Start',
    'add' => 'Import',
    'choose-initial-text-for-export' => 'ChooseInitialText',
    'choose-initial-text-for-import' => 'ChooseInitialText',
    'delete' => 'Delete',
    'download' => 'Download',
    'export' => 'Export',
    'show-edit-window' => 'ShowEditWindow',
    'show-options-for-add' => 'ShowOptions',
    'show-options-for-delete' => 'ShowOptions',
    'show-options-for-update' => 'ShowOptions',
    'start' => 'Start',
    'update' => 'Import' );
  if ( in_array( $task, array_keys( $tasks ) ) )
  {
    call_user_func( '_Userport_' . $tasks[ $task ], $option, $task, $settings, $log );
  }
  else
  {
    $log[] = JText::sprintf( 'ERROR_INVALID_TASK', $task );
    JUserportHtml::PrintLog( JText::_( 'FORM_LOG' ), $log );
    JUserportHtml::DummyForm( $option, $task );
  }
}

/******************************************************************************/

$log = array();
$settings = JUserportHelper::LoadSettings();
_Userport_HandleTask( $option, $task, $settings, $log );
?>
