<?php
/*
 * @package userport
 * @copyright 2008-2011 Parvus
 * @license http://www.gnu.org/licenses/gpl-3.0.html
 * @link http://joomlacode.org/gf/project/userport/
 * @author Parvus
 *
 * userport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * userport is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with userport. If not, see <http://www.gnu.org/licenses/>.
 */
defined( '_JEXEC' ) or die( 'Restricted access' );

class JUserportToolbar
{
  /* Function DoSomething is called when the user choose the task
   * do_something, and the necessary actions have been carried out
   * Most of the time this means that Something has been Done.
   * Even though the verbs are in present time, they should
   * be interpreted here as past.
   */

  /**
   * Generic function which displays the buttons in the toolbar when all steps
   * have been taken by the user, and the task has been fully carried out by the
   * server.
   * @param $title The not-yet-translated title to be shown at the top of the page
   * @param $task Mandatory if $GLOBALS[ 'userport_next_batch_needs_processing' ] is set.
   * The task name that was prematurely stopped to prevent reaching the maximum
   * php execution time
   * @return void
   * @access private
   */
  function _ButtonsWhenFinished( $title, $task = '' )
  {
    JToolbarHelper::title( JText::_( $title ), 'userport' );
    if ( $task and isset( $GLOBALS[ 'userport_next_batch_needs_processing' ] ) )
    {
      JToolBarHelper::custom( 'add', 'forward', 'forward', JText::_( 'BUTTON_CONTINUE_PROCESSING' ), false, false );
    }
    JToolBarHelper::divider();
    $bar = & JToolBar::getInstance('toolbar');
    $bar->appendButton( 'Link', 'user', 'User Manager', 'index.php?option=com_users' );
    JToolBarHelper::custom( 'start', 'start', 'start', JText::_( 'BUTTON_START' ), false, false );
  }

  /**
   * Generic function which displays the buttons in the toolbar when all steps
   * but one have been taken by the user, and that all that is left is choosing
   * the task related options.
   * @param $title The not-yet-translated title to be shown at the top of the
   * page.
   * @param $nextTask The (final) task to be carried out after choosing the
   * options.
   * @param $buttonText The not-yet-translated text to show below the (final)
   * task.
   * @access private
   */
  function _ButtonsWhenShowingOptions( $title, $nextTask, $buttonText )
  {
    JToolbarHelper::title( JText::_( $title ), 'userport' );
    JToolBarHelper::custom( $nextTask, $nextTask, $nextTask, $buttonText, false, false );
    JToolBarHelper::divider();
    JToolBarHelper::back();
    JToolBarHelper::custom( 'start', 'start', 'start', JText::_( 'BUTTON_START' ), false, false );
  }

  function Export()
  {
    JToolBarHelper::custom( 'download', 'download', 'download', JText::_( 'BUTTON_DOWNLOAD' ), false, false );
    JUserportToolbar::_ButtonsWhenFinished( 'TITLE_EXPORT' );
  }

  function Add()
  {
    JUserportToolbar::_ButtonsWhenFinished( 'TITLE_ADD', 'add' );
  }

  function Update()
  {
    JUserportToolbar::_ButtonsWhenFinished( 'TITLE_UPDATE', 'update' );
  }

  function Delete()
  {
    JUserportToolbar::_ButtonsWhenFinished( 'TITLE_DELETE', 'delete' );
  }

  function ShowOptionsForAdd()
  {
    JUserportToolbar::_ButtonsWhenShowingOptions( 'TITLE_OPTIONS_FOR_ADD', 'add', 'BUTTON_ADD' );
  }

  function ShowOptionsForUpdate()
  {
    JUserportToolbar::_ButtonsWhenShowingOptions( 'TITLE_OPTIONS_FOR_UPDATE', 'update', 'BUTTON_UPDATE' );
  }

  function ShowOptionsForDelete()
  {
    JUserportToolbar::_ButtonsWhenShowingOptions( 'TITLE_OPTIONS_FOR_DELETE', 'delete', 'BUTTON_DELETE' );
  }


  function ShowEditWindow()
  {
    JToolbarHelper::title( JText::_( 'TITLE_SHOW_EDIT_WINDOW' ), 'userport' );
    JToolBarHelper::custom( 'download', 'download', 'download', JText::_( 'BUTTON_DOWNLOAD' ), false, false );
    JToolBarHelper::custom( 'show-options-for-add', 'add', 'add', JText::_( 'BUTTON_ADD' ) . "...", false, false );
    JToolBarHelper::custom( 'show-options-for-update', 'update', 'update', JText::_( 'BUTTON_UPDATE' ) . "...", false, false );
    JToolBarHelper::custom( 'show-options-for-delete', 'delete', 'delete', JText::_( 'BUTTON_DELETE' ) . "...", false, false );
    JToolBarHelper::divider();
    JToolBarHelper::back();
    JToolBarHelper::custom( 'start', 'start', 'start', JText::_( 'BUTTON_START' ), false, false );
  }

  function ChooseInitialTextForExport()
  {
    JUserportToolbar::_ButtonsWhenShowingOptions( 'TITLE_OPTIONS_FOR_EXPORT', 'export', 'export', 'BUTTON_EXPORT' );
  }

  function ChooseInitialTextForImport()
  {
    JToolbarHelper::title( JText::_( 'TITLE_CHOOSE_INITIAL_TEXT' ), 'userport' );
    JToolBarHelper::custom( 'show-edit-window', 'forward', 'forward', JText::_( 'BUTTON_SHOW_EDIT_WINDOW' ), false, false );
    JToolBarHelper::divider();
    JToolBarHelper::custom( 'start', 'start', 'start', JText::_( 'BUTTON_START' ), false, false );
  }

  function Start()
  {
    JToolbarHelper::title( JText::_( 'TITLE_START' ), 'userport' );
    JToolBarHelper::custom( 'choose-initial-text-for-import', 'import', 'import', JText::_( 'BUTTON_CHOOSE_INTIAL_TEXT' ), false, false );
    JToolBarHelper::custom( 'choose-initial-text-for-export', 'export', 'export', JText::_( 'BUTTON_EXPORT' ), false, false );
    JToolBarHelper::divider();
    $bar = & JToolBar::getInstance('toolbar');
    $bar->appendButton( 'Link', 'user', 'User Manager', 'index.php?option=com_users' );
    JToolBarHelper::preferences( 'com_userport', 400, 575 );
  }
}

?>
