<?php
/*
 * @package userport
 * @copyright 2008-2011 Parvus
 * @license http://www.gnu.org/licenses/gpl-3.0.html
 * @link http://joomlacode.org/gf/project/userport/
 * @author Parvus
 *
 * userport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * userport is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with userport. If not, see <http://www.gnu.org/licenses/>.
 */
defined( '_JEXEC' ) or die( 'Restricted access' );

jimport( 'joomla.filesystem.file' );
require_once( JPATH_LIBRARIES . DS . 'phputf8' . DS . 'utils' . DS . 'validation.php' );

$user = & JFactory::getUser();
$user->authorize( 'com_users', 'manage' ) or die( JText::_( 'ALERTNOTAUTH' ) );

class JUserportHelper
{
  /**
   * Sends an email to the affected user.
   * Given the just performed task, a user will be sent an email, based upon the
   * templates as set in the preferences.
   * @param $task A string. May be one of 'add', 'update' or 'delete'. Other
   * values will not trigger an email.
   * @param $user a JUser instance. References the user that was affected.
   * May not be Null.
   * @param $userLog An array of strings. Gives detailed information what has
   * been changed, and can get inserted in the email.
   * @return boolean True when an email was successfully sent.
   */
  function NotifyUser( $task, $user, $settings, $userLog = null )
  {
    $success = false;
    global $mainframe;
    $admin =& JFactory::getUser();

    if ( $settings[ 'dry_run' ] )
    {
      /* Only send out emails to the 'dry run' email address. */
      $recipients = array( '{cc}' => '', '{bcc}' => '' );
    }
    else
    {
      $recipients = array( '{cc}' => $settings[ 'cc' ], '{bcc}' => $settings[ 'bcc' ] );
    }
    foreach( array_keys( $recipients ) as $r )
    {
      if ( JString::strlen( $recipients[ $r ] ) == 0 )
      {
        $recipients[ $r ] = null;
      }
      else
      {
        $recipients[ $r ] = explode( ',', $recipients[ $r ] );
      }
    }

    if ( isset( $userLog ) )
    {
      if ( JString::strlen( $userLog ) == 0 )
      {
        $userLog = null;
      }
      else
      {
        $userLog = "\n- " . implode( "\n- ", $userLog );
      }
    }

    $activationUrl = JURI::root();
    if ( JString::strlen( $user->activation ) > 0 )
    {
      $activationUrl .= "index.php?option=com_user&task=activate&activation=" . $user->activation;
    }
    $token = array
    ( '{user_name}' => $user->name,
      '{user_login}' => $user->username,
      '{user_password}' => $user->password,
      '{user_type}' => $user->usertype,
      '{site_name}' => $mainframe->getCfg( 'sitename' ),
      '{site_url}' => JURI::root(),
      '{admin_name}' => $admin->name,
      '{admin_email}' => $admin->email,
      '{user_log}' => $userLog,
    	'{activation_url}' => $activationUrl );

    $email = array( '{email_subject}' => $settings[ 'email_subject' ], '{email_body}' => $settings[ 'email_body' ] );
    foreach( array_keys( $email ) as $e )
    {
      foreach( array_keys( $token ) as $t )
      {
        $email[$e] = JString::str_ireplace( $t, $token[$t], $email[$e] );
      }
      $email[$e] = html_entity_decode( $email[$e], ENT_QUOTES );
    }

    if ( $settings[ 'dry_run' ] )
    {
      /* Only send out emails to the 'dry run' email address. */
      $to = $settings[ 'dry_run_email' ];
    }
    else
    {
      $to = $user->email;
    }
    $to = JString::trim( $to );

    if ( ( JString::strlen( $email['{email_subject}'] ) > 0 )
        and ( JString::strlen( $email['{email_body}'] ) > 0 )
        and ( JString::strlen( $to ) > 0 ) )
    {
      $success = JUtility::sendMail
      ( /* from */ $token[ '{admin_email}' ],
        /* fromname */ $token[ '{admin_name}' ],
        /* recipient */ $to,
        /* subject */ $email[ '{email_subject}' ],
        /* body */ $email[ '{email_body}' ],
        /* mode */ 0,
        /* cc */ $recipients[ '{cc}' ],
        /* bcc */ $recipients[ '{bcc}' ],
        /* attachment */ null,
        /* replyto */ null,
        /* replytoname */ null);
    }
    return $success;
  }

  /**
   * Converts a CSV string to an array.
   * The given string is split in fields. An entry 'superfluous' will be created
   * iff there are more fields than heades; and if there are less fields than
   * headers, empty fields are added.
   * @param $string A string. One line of text from which the fields should be
   * extracted.
   * @param $headerData An ordered unnamed array. It is assumed that
   * the field types correspond (this is not checked here).
   * If this value is @c null, the string data is also used as variable names
   * in the resulting object.
   * @param $fieldSeparator A string. The character (can be a full string as
   * well) that is used separating two fields.
   * @param $settings
   * @param $log An array of arrays of strings. Used for logging information
   * messages, warnings and errors, interesting for the user of userport.
   * @return An object, with the headerdData as variable names, and the string
   * data as the variable values. The field values in the array are trimmed.
   */
  function CsvStringToObject( $string, $headerData, $fieldSeparator, $settings, & $log )
  {
    if ( $fieldSeparator == 'tab' )
    {
      $fieldSeparator = "\t";
    }
    if ( function_exists( 'str_getcsv' ) )
    {
      $fields = str_getcsv( $string, $fieldSeparator );
    }
    else
    {
      $fields = explode( $fieldSeparator, $string );
    }
    if ( !/*NOT*/ $headerData )
    {
      $headerData = $fields;
      /* Do not give the same warning on each and every line. Only check for
       * existence when decoding eth header data.
       */
      if ( !/*NOT*/ function_exists( 'str_getcsv' ) )
      {
        $log[] = array( 'warning', JText::sprintf( 'WARNING_LIMITED_CSV_IMPORT_FUNCTIONALITY', 'str_getcsv' ) );
      }
    }
    while ( count( $fields ) < count( $headerData) )
    {
      $fields[] = '';
    }
    $userData = array();
    for ( $i = 0; $i < count( $headerData ); $i++ )
    {
      $headerData[ $i ] = JString::trim( $headerData[ $i ] );
      if ( JString::strlen( $headerData[ $i ] ) > 0 )
      {
        $fields[ $i ] = JString::trim( $fields[ $i ] );
        $userData[ $headerData[ $i ] ] = $fields[ $i ];
      }
    }
    if ( count( $fields ) > count( $headerData) )
    {
      $userData[ 'superfluous' ] = implode( $fieldSeparator, array_slice( $fields, count( $headerData ) ) );
    }
    return (object)$userData;
  }

  /**
   * Converts an object to a CSV string.
   * The given array is concatenated into a string.
   * @param $userData An object (preferably of class JUser) containing the user's data,
   * which must be transformed to a string. If @c null, a line with the field names used
   * will be constructed, which can serve as a header line.
   * @param $fieldSeparator A string. The character (can be a full string as
   * well) that is used separating two fields.
   * @param $settings
   * @param $log An array of arrays of strings. Used for logging information
   * messages, warnings and errors, interesting for the user of userport.
   * @return A string. Contains the contents of the array, separated by the
   * given separator.
   * @todo add checks on validity of email addresses
   */
  function ObjectToCsvString( $userData, $fieldSeparator, $settings, & $log )
  {
    $include = JUserportHelper::Fields( $settings[ 'which_fields' ] );

    $data = array();
    foreach( $include as $var )
    {
      if ( !/*NOT*/isset( $userData ) )
      {
        $data[] = $var;
      }
      elseif ( isset( $userData->$var ) )
      {
        if ( JString::strpos( $userData->$var, $fieldSeparator ) !== false )
        {
          $log[] = array( 'warning', JText::sprintf( 'WARNING_SPECIAL_CHAR_REPLACED_IN_FIELD', $userData->$var, $var, $fieldSeparator, '_' ) );
          $userData->$var = JString::str_ireplace( $fieldSeparator, '_', $userData->$var );
        }
        if ( JString::strpos( $userData->$var, '|' ) )
        {
          $log[] = array( 'warning', JText::sprintf( 'WARNING_SPECIAL_CHAR_REPLACED_IN_FIELD', $userData->$var, '|' ) );
          $userData->$var = JString::str_ireplace( '|', '_', $userData->$var );
        }
        $userData->$var = JString::str_ireplace( "\n", '|', $userData->$var );
        $data[] = $userData->$var;
      }
      else
      {
        $data[] = '';
      }
    }

    if ( $fieldSeparator == 'tab' )
    {
      $fieldSeparator = "\t";
    }
    else
    {
      $fieldSeparator .= ' ';
    }
    $string = implode( $fieldSeparator, $data) . "\n";
    return $string;
  }

  /**
   * Determines the most likely field separator, given a string.
   * @param $string @pre The string is assumed to be the header of an import string.
   * This matters: the header can only contain azAZ-_ and the simple logic employed
   * exploits this.
   * @return A character: the assumed field separaror.
   */
  function DetermineFieldSeparator( $string )
  {
    /* Keep the list of possible field separators limited - the more,
     * the more chance on false positives.
     * It is also possible no field separator is found.
     * For e.g. deleting only a username is sufficient.
     * So a warning or error can't be generated in that case.
     * This test will fail on strings like
     *   "a,b";"c,d"
     * But those strings can't possibly form a valid header anyway.
     */

    if ( JString::strpos( $string, "\t" ) !== false )
    {
      $fieldSeparator = "tab";
    }
    else if ( JString::strpos( $string, ';' ) !== false )
    {
      $fieldSeparator = ';';
    }
    else
    {
      $fieldSeparator = ',';
    }
    return $fieldSeparator;
  }

  /**
   * Returns a list of user types that are allowed to be manipulated by userport.
   * @note order matches with @c Gids()
   * @return An array.
   */
  function UserTypes()
  {
    return array( 'Registered', 'Author', 'Editor', 'Publisher' );
  }

 /**
   * Returns a list of gids that are allowed to be manipulated by userport.
   * @note order matches with @c UserTypes()
   * @return An array.
   */
  function Gids()
  {
    return array( 18, 19, 20, 21 );
  }

  /**
   * Returns the gid integer value of the given user type. If the user type is not valid,
   * the Registered gid is returned.
   * @param $userType A string. The user type.
   * @return An integer
   */
  function Gid( $userType )
  {
    $usertypeToGid = array( 'Registered'=>18, 'Author'=>19, 'Editor'=>20, 'Publisher'=>21 );
    if ( !/*NOT*/array_key_exists( $userType, $usertypeToGid ) )
    {
      $userType = 'Registered';
    }
    return $usertypeToGid[ $userType ];
  }

  /**
   * Returns a list of user table fields that are allowed to be manipulated by userport.
   * @param $which Either 'basic' to include only the most relevant information;
   * either 'import' to include all fields needed for a successful import in another database.
   * or 'all' to include all fields.
   * @return An array.
   */
  function Fields( $which )
  {
    /* Assumption is that block is listed before lastvisitDate, if they are present.
     * See _UpdateUser_block().
     */
    switch ( $which )
    {
      case 'basic':
        $fields = array( 'name', 'username', 'email', 'password' );
        break;

      case 'import':
        $fields = array( 'name', 'username', 'email', 'password', 'usertype', 'block', 'sendEmail', 'registerDate', 'lastvisitDate', 'activation', 'params' );
        break;

      case 'all':
      default:
        $fields = array( 'id', 'name', 'username', 'email', 'password', 'usertype', 'block', 'sendEmail', 'gid', 'registerDate', 'lastvisitDate', 'activation', 'params' );
        break;
    }
    return $fields;
  }

  /**
   * Queries the database to retrieve a listing of (a subset of) the userlist
   * @param $settings
   * @param $log
   * @return An array of objects; each object represents a user. All
   * table fields are available.
   */
  function GetUserList( $task, $settings, & $log )
  {
    $queryCondition = array(
      'nonblocked-and-nonactivated' => " AND ((block = '0') AND (activation != ''))",
      'nonblocked-and-   activated' => " AND ((block = '0') AND (activation = ''))",
      'nonblocked-and-            ' => " AND (block = '0')",
      'nonblocked- or-nonactivated' => " AND ((block = '0') OR (activation != ''))",
      'nonblocked- or-   activated' => " AND ((block = '0') OR (activation = ''))",
      'nonblocked- or-            ' => " AND (block = '0')",
      '   blocked-and-nonactivated' => " AND ((block = '1') AND (activation != ''))",
      '   blocked-and-   activated' => " AND ((block = '1') AND (activation = ''))",
      '   blocked-and-            ' => " AND (block = '1')",
      '   blocked- or-nonactivated' => " AND ((block = '1') OR (activation != ''))",
      '   blocked- or-   activated' => " AND ((block = '1') OR (activation = ''))",
      '   blocked- or-            ' => " AND (block = '1')",
      '          -and-nonactivated' => " AND (activation != '')",
      '          -and-   activated' => " AND (activation = '')",
      '          -and-            ' => " ",
      '          - or-nonactivated' => " AND (activation != '')",
      '          - or-   activated' => " AND (activation = '')",
      '          - or-            ' => "");
    $logCondition = array(
      'nonblocked-and-nonactivated' => array( 'INFO_ONLY_NON_BLOCKED_USERS_ARE_INCLUDED', 'INFO_ONLY_NON_ACTIVATED_USERS_ARE_INCLUDED' ),
      'nonblocked-and-   activated' => array( 'INFO_ONLY_NON_BLOCKED_USERS_ARE_INCLUDED', 'INFO_ONLY_ACTIVATED_USERS_ARE_INCLUDED' ),
      'nonblocked-and-            ' => array( 'INFO_ONLY_NON_BLOCKED_USERS_ARE_INCLUDED' ),
      'nonblocked- or-nonactivated' => array( 'INFO_ONLY_NON_BLOCKED_OR_NON_ACTIVATED_USERS_ARE_INCLUDED' ),
      'nonblocked- or-   activated' => array( 'INFO_ONLY_NON_BLOCKED_OR_ACTIVATED_USERS_ARE_INCLUDED' ),
      'nonblocked- or-            ' => array( 'INFO_ONLY_NON_BLOCKED_USERS_ARE_INCLUDED' ),
      '   blocked-and-nonactivated' => array( 'INFO_ONLY_BLOCKED_USERS_ARE_INCLUDED', 'INFO_ONLY_NON_ACTIVATED_USERS_ARE_INCLUDED' ),
      '   blocked-and-   activated' => array( 'INFO_ONLY_BLOCKED_USERS_ARE_INCLUDED', 'INFO_ONLY_ACTIVATED_USERS_ARE_INCLUDED' ),
      '   blocked-and-            ' => array( 'INFO_ONLY_BLOCKED_USERS_ARE_INCLUDED' ),
      '   blocked- or-nonactivated' => array( 'INFO_ONLY_BLOCKED_OR_NON_ACTIVATED_USERS_ARE_INCLUDED' ),
      '   blocked- or-   activated' => array( 'INFO_ONLY_BLOCKED_OR_ACTIVATED_USERS_ARE_INCLUDED' ),
      '   blocked- or-            ' => array( 'INFO_ONLY_BLOCKED_USERS_ARE_INCLUDED' ),
      '          -and-nonactivated' => array( 'INFO_ONLY_NON_ACTIVATED_USERS_ARE_INCLUDED' ),
      '          -and-   activated' => array( 'INFO_ONLY_ACTIVATED_USERS_ARE_INCLUDED' ),
      '          -and-            ' => array(),
      '          - or-nonactivated' => array( 'INFO_ONLY_NON_ACTIVATED_USERS_ARE_INCLUDED' ),
      '          - or-   activated' => array( 'INFO_ONLY_ACTIVATED_USERS_ARE_INCLUDED' ),
      '          - or-            ' => array() );
    $condition = '';
    switch ( $settings[ 'block' ] )
    {
      case '0': $condition .= 'nonblocked'; break;
      case '1': $condition .= '   blocked'; break;
      default : $condition .= '          '; break;
    }
    $condition .= '-';
    switch ( $settings[ 'filter_combination' ] )
    {
      case 'and': $condition .= 'and'; break;
      default : $condition .= ' or'; break;
    }
    $condition .= '-';
    switch ( $settings[ 'non_activated' ] )
    {
      case '0': $condition .= '   activated'; break;
      case '1': $condition .= 'nonactivated'; break;
      default : $condition .= '            '; break;
    }

    $db = & JFactory::getDBO();
    $query = "SELECT * FROM #__users";
    if ( $task == 'export' )
    {
      /* Retain all users. */
      $user = & JFactory::getUser();
    }
    else
    {
      /* Only retain front-end-only users. */
      $query .= " WHERE (gid = '18' OR gid = '19' OR gid = '20' OR gid = '21')";
      $log[] = array( 'info', JText::_( 'INFO_FRONT_END_USERS_ONLY' ) );
    }
    $query .= $queryCondition[ $condition ];

    $db->setQuery( $query );
    $rows = $db->loadObjectList();
    if ( $db->getErrorNum() )
    {
      $log[] = array( 'error', JText::sprintf( 'ERROR_COULD_NOT_ACCESS_DATABASE', JText::_( $db->stderr() ) ) );
    }
    $log[] = array( 'info', JText::sprintf( 'INFO_RETRIEVED_USER_COUNT', count( $rows ) ) );
    foreach ( $logCondition[ $condition ] as $info )
    {
      $log[] = array( 'info', JText::_( $info ) );
    }

    return $rows;
  }

  /**
   * Presents a download file to the user, so he can store the given text
   * under the suggested filename locally on his PC.
   * @param $text String. The contents of the file to download.
   * @param $filename String. The suggested filename.
   */
  function CreateDownloadFile( $text, $filename )
  {
    $mimeType = 'text/x-csv';
    if ( ini_get( 'zlib.output_compression' ) )
    {
      /* Required for IE. Content-Disposition may get ignored otherwise. */
      ini_set( 'zlib.output_compression', 'Off' );
    }

    header( 'Content-Disposition: attachment; filename=' . $filename );
    header( 'Content-Transfer-Encoding: UTF-8' );
    header( 'Content-Type: ' . $mimeType );

    /* Make the download non-cacheable. */
    header( 'Expires: ' . gmdate('D, d M Y H:i:s') . ' GMT' );
    header( 'Cache-control: private' );
    header( 'Pragma: private' );

    echo $text;
    flush();

    global $mainframe;
    $mainframe->close();
  }


  /**
   * Reads out the entire contents of the file on the server. Performs some basic checks to ensure
   * the text is read well.
   * @param $filenameOnServer The file to read out
   * @param $log An array of arrays of strings. Used for logging information
   * messages, warnings and errors, interesting for the user of userport.
   * @return A string. The contents of the file when all went succesfull.
   */
  function ReadFile( $filename, & $log )
  {
    $text = JFile::read( $filename );
    if ( $text === false )
    {
      $log[] = array( 'error', JText::_( 'ERROR_READING_UPLOADED_FILE' ) );
      $text = '';
    }
    else if ( JString::strlen( $text ) == 0 )
    {
      $log[] = array( 'error', JText::_( 'ERROR_FILE_EMPTY' ) );
      $text = '';
    }
    else if ( function_exists( 'mb_detect_encoding' )
        and function_exists( 'mb_convert_encoding' ) )
    {
      if ( !/*NOT*/utf8_is_valid( $text ) )
      {
        $encoding = mb_detect_encoding( $text . "a" );
        /* The encoding detection can get it wrong. It can report that
         * the text is UTF-8 even when we know @c utf8_is_valid() fails.
         */
        if ( !/*NOT*/$encoding or ( $encoding == 'UTF-8' ) )
        {
          $text = utf8_encode( $text );
        }
        else
        {
          $text = mb_convert_encoding( $text, 'UTF-8', $encoding );
        }
      }
    }
    else
    {
      /* Assume all is OK. */
    }

    return $text;
  }

  /**
   * Reads out the entire contents of the uploaded file.
   * @param $log An array of arrays of strings. Used for logging information
   * messages, warnings and errors, interesting for the user of userport.
   * @param  $removeQuotes Bool. If true, all quotes " and ' are stripped from the uploaded text.
   * @return A string. The contents of the file when all went succesfull.
   */
  function GetUploadedFile( & $log, $removeQuotes )
  {
    $file = JRequest::getVar('file_upload', null, 'files', 'array');
    $filenameOnClientPc = $file['name'];
    $filenameOnServer = $file['tmp_name'];
    $text = '';

    $log[] = array( 'info', JText::sprintf( 'INFO_SELECTED_FILE_TO_UPLOAD', $filenameOnClientPc ) );

    if ( JString::strlen( $filenameOnClientPc ) == 0 )
    {
      $log[] = array( 'error', JText::_( 'ERROR_NO_FILE_TO_UPLOAD' ) );
      $isTextFile = false;
    }
    else if ( JString::strlen( $filenameOnServer ) == 0 )
    {
      $log[] = array( 'error', JText::sprintf( 'ERROR_UPLOADING_FILE', $filenameOnClientPc ) );
      $isTextFile = false;
    }
    else if ( function_exists( 'finfo_open' ) and function_exists( 'finfo_file' )
        and function_exists( 'finfo_close' ) )
    {
      $finfo = finfo_open( FILEINFO_MIME );
      $mime = finfo_file( $finfo, $filenameOnServer );
      finfo_close( $finfo );
      $isTextFile = ( JString::strpos( $mime, 'text/plain' ) !== FALSE );
    }
    else
    {
      /* Assume all is OK. */
      $isTextFile = true;
    }

    if ( $isTextFile )
    {
      $text = self::ReadFile( $filenameOnServer, $log );
      if ( $removeQuotes )
      {
        if ( ( JString::strpos( $text, '"' ) !== false )
            or ( JString::strpos( $text, "'" ) !== false ) )
        {
          $log[] = array( 'warning', JText::_( 'WARNING_REMOVED_QUOTES' ) );
        }
        $text = JString::str_ireplace( '"', '', $text );
        $text = JString::str_ireplace( "'", "", $text );
      }
    }
    else
    {
      $log[] = array( 'error', JText::_( 'ERROR_FILE_INVALID_TYPE' ) );
    }

    return $text;
  }

  /**
   * Get all settings.
   * @return A named array (hash). The values can be a string, a number or a boolean;
   * this depends on the key and is assumed to be known.
   * @return void
   */
  function LoadSettings()
  {
    $settings = array();
    jimport( 'joomla.application.component.helper' );
    $params =& JComponentHelper::getParams( 'com_userport' );

    $vars = array(
      'activation' => '',
      'which_fields' => 'import',
      'auto_generate_passwords' => true,
      'bcc' => '',
    	'block' => false,
      'block_activate' => 'default',
      'cc' => '',
      'confirmation1' => false,
      'confirmation2' => false,
      'confirmation3' => false,
      'default_usertype' => 'Registered',
      'download' => false,
      'dry_run' => true,
      'dry_run_email' => '',
      'email_body' => '',
      'email_subject' => '',
      'field_separator' => ';',
      'filter_combination' => 'or',
      'initial_text' => 'empty',
      'lastvisitDate' => '',
      'non_activated' => true,
      'notify_user' => true,
      'overwrite_password' => false,
      'params' => '',
      'parsing_index' => 0,
      'passwords_are_encoded' => true,
      'registerDate' => '',
      'sendEmail' => 'default',
      'user_list' => '',
      'usertype' => 'default',
     	'remove_quotes' => false );
    foreach ( $vars as $var => $value )
    {
      $settings[ $var ] = JRequest::getVar( $var, $params->get( $var, $value ), 'post', '' );
    }
    $settings[ 'user_list' ] = urldecode( $settings[ 'user_list' ] );

    return $settings;
  }
}

?>
